/*******************************************************************************
*    IM-Daemon                                                                 *
*    Copyright (C) 2012-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef IM_H_
#define IM_H_

#include <account.h>
#include <conversation.h>
#include <core.h>
#include <debug.h>
#include <eventloop.h>
#include <ft.h>
#include <log.h>
#include <notify.h>
#include <prefs.h>
#include <prpl.h>
#include <pounce.h>
#include <savedstatuses.h>
#include <sound.h>
#include <status.h>
#include <util.h>
#include <whiteboard.h>

#define CUSTOM_USER_DIRECTORY  "/dev/null"
#define CUSTOM_PLUGIN_PATH     "/dev/null"
#define PLUGIN_SAVE_PREF       "/dev/null"
#define UI_ID                  "IM-Daemon"

#define PURPLE_GLIB_READ_COND  (G_IO_IN | G_IO_HUP | G_IO_ERR)
#define PURPLE_GLIB_WRITE_COND (G_IO_OUT | G_IO_HUP | G_IO_ERR | G_IO_NVAL)

typedef struct _PurpleGLibIOClosure
{
  PurpleInputFunction function;
  guint result;
  gpointer data;
} PurpleGLibIOClosure;

/**
 * Nötige Funktion für die Verwaltung der Meldungen vom Server
 */
void glibIODestroy(gpointer data);

/**
 * Nötige Funktion für die Verwaltung der Meldungen vom Server
 */
gboolean glibIOInvoke(GIOChannel *source, GIOCondition condition, gpointer data);

/**
 * Nötige Funktion für die Verwaltung der Meldungen vom Server
 */
guint glibInputAddCB(gint fd, PurpleInputCondition condition, PurpleInputFunction function, gpointer data);

/**
 * Initialisiert die LibPurple
 */
void initPurple(void);

/**
 * Setzt die eigene Icon
 *
 * @param PurpleAccount *account          Pidgin-Konto
 * @param char *icon                      Pfad zur Icon
 */
void setMyIcon(PurpleAccount *account, char *icon);

/**
 * CallBack nach dem erfolgreichen Login
 */
void signedOnCB(PurpleConnection *gc, gpointer null);

/**
 * CallBack für die Anfrage nach Autorisierung
 */
int authRequestedCB(PurpleAccount *account, const char *user);

/**
 * CallBack nach einer Trennung des Netzes
 */
void networkDisconnectedCB(void);

/**
 * CallBack nach einer Trennung des Netzes
 */
void reportDisconnectReasonCB(PurpleConnection *gc, PurpleConnectionError reason, const char *reasonText);

/**
 * Verbindet mit dem Server
 *
 * @param struct config cfg     Die Konfiguration
 */
void connectToIMServer(struct config cfg);

/**
 * Trennt die Verbindung vom Server
 */
void disconnectFromIMServer(void);

/**
 * Schicke eine IM-Meldung einem Empfängern
 *
 * @param char *recipient       Der Empfänger
 * @param char *message         Die Meldung
 */
void sendIM(char *recipient, char *message);

/**
 * Gibt zurück, ob die Verbindung mit dem Server besteht
 *
 * @return gboolean             TRUE wenn die Verbindung besteht,
 *                              FALSE wenn nicht
 */
gboolean isSignedOn(void);

/**
 * Gibt zurück, ob gerade versucht wird, sich mit dem Server zu verbinden
 *
 * @return gboolean             TRUE wenn eine Verbingung aufgebaut wird,
 *                              FALSE wenn nicht
 */
gboolean isConnecting(void);

#endif /* IM_H_ */
