/*******************************************************************************
*    IM-Daemon                                                                 *
*    Copyright (C) 2012-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef LOG_H_
#define LOG_H_

#define LOGL_DEBUG         0 // Debug
#define LOGL_INFO          1 // Allgemeine Informationen
#define LOGL_NOTICE        2 // Schon was unnormales
#define LOGL_ERROR         3 // Fehler, aber geht noch weiter
#define LOGL_FATAL         4 // Fehler, Programm abgebrochen

#define MAX_LOG_LENGTH  1024

#define LOGGER(ss, fmt, args...) logger(ss, __FILE__, __LINE__, fmt, ## args)

/**
 * Setze die aktuelle Logstufe
 *
 * @param int level       Logstufe (DEBUG, INFO, usw.)
 */
void setLogLevel(int level);

/**
 * Setze die aktuelle Logdatei
 *
 * @param char *file      Die Datei, wo die Logevente gespeichert werden
 */
void setLogFile(char *file);

/**
 * Initialisiert das Logsystem
 */
void openLogger(void);

/**
 * Schließt das Logsystem
 */
void closeLogger(void);

/**
 * Loggt auf Datei und, wenn eingestellt, auf der Konsole
 *
 * @param int level       Loglevel (LOGL_DEBUG, LOGL_INFO, usw.)
 * @param char *file      Die Datei, wo diese Meldung generiert worden ist
 * @param int line        Die Zeile, wo diese Meldung generiert worden ist
 * @param char *format    Die Meldung, formatiert wie bei printf
 */
void logger(int level, char *file, int line, const char *format, ...)
        __attribute__ ((format (printf, 4, 5)));

#endif /* LOG_H_ */
