/*******************************************************************************
*    IM-Daemon                                                                 *
*    Copyright (C) 2012-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "incs/constants.h"
#include "incs/options.h"

/**
 * Liest die Optionen vom Kommandozeile
 *
 * @param int            argc  Anzahl der Parameter im Kommandozeile
 * @param char*          argv  Die Parameter im Kommandozeile
 * @param struct options opts  Die Beschreibung der möglichen Parameter
 */
int getOptions(int argc, char* argv[], struct options opts)
{
  int l, k, found, invalidOption;
  char *opt, *par;
  char *tOptLong, tOptShort[3];

  for(invalidOption = k = 0; k < opts.nOptions; k++)
  {
    if(opts.opt[k].type == INT)
    {
      if(opts.opt[k].defValue != NULL)
        *(int*)opts.opt[k].value = *(int*)opts.opt[k].defValue;
      else
        *(int*)opts.opt[k].value = 0;
    }
    if(opts.opt[k].type == BOOL)
    {
      if(opts.opt[k].defValue != NULL)
        *(gboolean*)opts.opt[k].value = *(gboolean*)opts.opt[k].defValue;
      else
        *(gboolean*)opts.opt[k].value = 0;
    }
    if(opts.opt[k].type == STRING)
    {
      if(opts.opt[k].defValue != NULL)
        strcpy(opts.opt[k].value, opts.opt[k].defValue);
      else
        strcpy(opts.opt[k].value, "");
    }
  }
  for(l = 1; l < argc; l++)
  {
    if(argv[l][0] == '-')
    {
      if(strstr(argv[l], "=") != NULL)
      {
        opt = strtok(argv[l], "=");
        par = strtok(NULL, "=");
      }
      else
      {
        opt = argv[l];
        par = NULL;
      }
      for(found = k = 0; k < opts.nOptions; k++)
      {
        tOptLong = malloc(sizeof(char) * (strlen(opts.opt[k].optNameLong) + 3));
        sprintf(tOptLong, "--%s", opts.opt[k].optNameLong);
        sprintf(tOptShort, "-%c", opts.opt[k].optNameShort);
        if((strcmp(tOptLong, opt) == 0) || (strcmp(tOptShort, opt) == 0))
        {
          found = 1;
          if(opts.opt[k].type == INT)
          {
            if(par == NULL)
            {
              par = argv[l + 1];
              l++;
            }
            *(int*)opts.opt[k].value = atoi(par);
          }
          if(opts.opt[k].type == STRING)
          {
            if(par == NULL)
            {
              par = argv[l + 1];
              l++;
            }
            strcpy(opts.opt[k].value, par);
          }
          if(opts.opt[k].type == BOOL)
          {
            *(gboolean*)opts.opt[k].value = 1;
          }
        }
        free(tOptLong);
      }
      if(found == 0)
      {
        fprintf(stderr, opts.invalidOptionMsg, opt);
        invalidOption++;
      }
    }
  }

  return(invalidOption);
}
